/*---------------------------------------------------------------------------*
 |          Copyright (c) 1997-2013 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 |           pCOS - PDF Information Retrieval Tool                           |
 *---------------------------------------------------------------------------*/

/* $Id: pcoslib.h,v 1.43.2.1 2013/08/05 17:21:29 rjs Exp $
 *
 * pCOS public function declarations
 *
 */

/*
 * ----------------------------------------------------------------------
 * Setup, mostly Windows calling conventions and DLL stuff
 * ----------------------------------------------------------------------
 */

#ifndef PCOSLIB_H
#define PCOSLIB_H

#include <stdio.h>


#if defined(WIN32) && !defined(PDFLIB_CALL)
#define PDFLIB_CALL     __cdecl
#endif

#undef PDFLIB_API
#if defined(WIN32)

    #ifdef PDFLIB_EXPORTS
    #define PDFLIB_API __declspec(dllexport) /* prepare a DLL (internal use) */

    #elif defined(PDFLIB_DLL)

    #define PDFLIB_API __declspec(dllimport) /* PDFlib clients: import  DLL */
    #endif	/* PDFLIB_DLL */

#else
    #if __GNUC__ >= 4
	#define PDFLIB_API __attribute__ ((visibility("default")))
    #elif (defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x550) \
	   || (defined(__SUNPRO_C) && __SUNPRO_C >= 0x550)
	#define PDFLIB_API __global
    #endif
#endif /* WIN32 */

#ifndef PDFLIB_CALL
    #define PDFLIB_CALL	/* */	/* default: no special calling conventions */
#endif

#ifndef PDFLIB_API
    #define PDFLIB_API /* */	 /* default: generate or use static library */
#endif

/* Make our declarations C++ compatible */
#ifdef __cplusplus
extern "C" {
#endif

#define PCOS_PRODUCTNAME         "pCOS"
#define PCOS_PRODUCTDESCR        "PDF Information Retrieval Tool"
#define PCOS_COPYRIGHT \
        "(c) 2007-2013 PDFlib GmbH  www.pdflib.com  sales@pdflib.com\n"

#define PCOS_MAJORVERSION		4
#define PCOS_MINORVERSION		0
#define PCOS_REVISION			0
#define PCOS_SHORT_VERSIONSTRING	"4"
#define PCOS_LONG_VERSIONSTRING		"4.0"

/* Opaque data type for the pCOS context. */
#if !defined(pCOS) || defined(ACTIVEX)
typedef struct pCOS_s pCOS;
#endif

/* The API structure with function pointers. */
typedef struct pCOS_api_s pCOS_api;

/*
 * ----------------------------------------------------------------------
 * pCOS-specific enums and defines
 * ----------------------------------------------------------------------
 */

/*
 * Guard against multiple definition of pcos_mode and pcos_object_type for the
 * case that multiple PDFlib products are used in the same program.
 */

#ifndef PDF_PCOS_ENUMS

/* document access levels.
*/
typedef enum
{
    pcos_mode_minimum	 = 0, /* encrypted doc (opened w/o password)	      */
    pcos_mode_restricted = 1, /* encrypted doc (opened w/ user password)      */
    pcos_mode_full	 = 2  /* unencrypted doc or opened w/ master password */
} pcos_mode;


/* object types.
*/
typedef enum
{
    pcos_ot_null	= 0,
    pcos_ot_boolean	= 1,
    pcos_ot_number	= 2,
    pcos_ot_name	= 3,
    pcos_ot_string	= 4,
    pcos_ot_array	= 5,
    pcos_ot_dict	= 6,
    pcos_ot_stream	= 7,
    pcos_ot_fstream	= 8
} pcos_object_type;

#define PDF_PCOS_ENUMS

#endif /* PDF_PCOS_ENUMS */


/*
 * ----------------------------------------------------------------------
 * pCOS API functions
 * ----------------------------------------------------------------------
 */

/* Release a document handle and all internal resources related to that
** document.
*/
PDFLIB_API void PDFLIB_CALL
pCOS_close_document(
    pCOS *p,
    int doc);

/* Convert a string in an arbitrary encoding to a Unicode string in various
   formats.
   Returns: The converted Unicode string.
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_convert_to_unicode(
    pCOS *p,
    const char *inputformat,
    const char *inputstring,
    int inputlen,
    int *outputlen,
    const char *optlist);

/* Create a named virtual read-only file from data provided in memory. */
PDFLIB_API void PDFLIB_CALL
pCOS_create_pvf(
    pCOS *p,
    const char *filename,
    int len,
    const void *data,
    size_t size,
    const char *optlist);

/* Deflate a Unicode string (UTF-16 or UTF-32) to a byte string (unsupported)
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_deflate_unicode(
    pCOS *p,
    const char *utfstring,
    int len,
    int charsize,
    int *highchar);

/* Delete a pCOS context and release all related internal resources. */
PDFLIB_API void PDFLIB_CALL
pCOS_delete(pCOS *p);

/* Delete a named virtual file and free its data structures (but not the
** contents).
** Returns: -1 if the virtual file exists but is locked, and
**           1 otherwise.
*/
PDFLIB_API int PDFLIB_CALL
pCOS_delete_pvf(
    pCOS *p,
    const char *filename,
    int len);

/*
 * Retrieve a structure with pCOS API function pointers (mainly for DLLs).
 * Although this function is published here, it is not supposed to be used
 * directly by clients. Use pCOS_new_dl() in pcosdl.c instead.
 */
PDFLIB_API const pCOS_api * PDFLIB_CALL
pCOS_get_api(void);

/* Get the name of the API function which threw the most recent exception or
** failed.
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_get_apiname(
    pCOS *p);

/* Get the descriptive text of the last thrown exception, or the reason of a
** failed function call.
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_get_errmsg(
    pCOS *p);

/* Get the number of the last thrown exception, or the reason for a failed
** function call.
*/
PDFLIB_API int PDFLIB_CALL
pCOS_get_errnum(
    pCOS *p);

/* Fetch the opaque client pointer from with a pCOS context. Unsupported. */
PDFLIB_API void * PDFLIB_CALL
pCOS_get_opaque(pCOS *p);

/* Query properties of a virtual file or the PDFlib Virtual Filesystem (PVF)
   Returns: The value of some file parameter as requested by keyword.
*/
PDFLIB_API double PDFLIB_CALL
pCOS_info_pvf(
    pCOS *p,
    const char *filename,
    int len,
    const char *keyword);

/* Create a new pCOS context. */
PDFLIB_API pCOS * PDFLIB_CALL
pCOS_new(void);

/* Create a new pCOS context with a user-supplied error handler and
** opaque pointer. Unsupported.
*/
typedef void (*pcos_error_fp)(pCOS *p, int type, const char *msg);

PDFLIB_API pCOS * PDFLIB_CALL
pCOS_new2(pcos_error_fp errorhandler, void *opaque);

/* Open a PDF document. */
PDFLIB_API int PDFLIB_CALL
pCOS_open_document(
    pCOS *p,
    const char *filename,
    int len,
    const char *optlist);

/* Deprecated, use pCOS_create_pvf()
 */
PDFLIB_API int PDFLIB_CALL
pCOS_open_document_mem(
    pCOS *p,
    const void *data,
    size_t size,
    const char *optlist);

/* Open a PDF document via a user-supplied function. */
PDFLIB_API int PDFLIB_CALL
pCOS_open_document_callback(
    pCOS *p,
    void *opaque,
    size_t filesize,
    size_t (*readproc)(void *opaque, void *buffer, size_t size),
    int (*seekproc)(void *opaque, long offset),
    const char *optlist);

/* Get the value of a pCOS path with type number or boolean. */
PDFLIB_API double PDFLIB_CALL
pCOS_pcos_get_number(
    pCOS *p,
    int doc,
    const char *path, ...);

/* Deprecated, use pCOS_pcos_get_number()
 */
PDFLIB_API double PDFLIB_CALL
pCOS_get_number(
    pCOS *p,
    int doc,
    const char *path, ...);

/* Get the value of a pCOS path with type name, string, or boolean. */
PDFLIB_API const char * PDFLIB_CALL
pCOS_pcos_get_string(
    pCOS *p,
    int doc,
    const char *path, ...);

/* Deprecated, use pCOS_pcos_get_string()
 */
PDFLIB_API const char * PDFLIB_CALL
pCOS_get_string(
    pCOS *p,
    int doc,
    const char *path, ...);

/* Get the contents of a pCOS path with type stream, fstream, or string. */
PDFLIB_API const unsigned char * PDFLIB_CALL
pCOS_pcos_get_stream(
    pCOS *p,
    int doc,
    int *length,
    const char *optlist,
    const char *path, ...);

/* Deprecated, use pCOS_pcos_get_stream()
 */
PDFLIB_API const unsigned char * PDFLIB_CALL
pCOS_get_stream(
    pCOS *p,
    int doc,
    int *length,
    const char *optlist,
    const char *path, ...);

/* Set one or more global options. */
PDFLIB_API void PDFLIB_CALL
pCOS_set_option(
    pCOS *p,
    const char *optlist);

/* Deprecated, use pCOS_convert_to_unicode().
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_utf8_to_utf16(
   pCOS *p,
   const char *utf8string,
   const char *ordering,
   int *size);

/* Deprecated, use pCOS_convert_to_unicode().
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_utf8_to_utf32(
    pCOS *p,
    const char *utf8string,
    const char *ordering,
    int *size);

/* Deprecated, use pCOS_convert_to_unicode().
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_utf16_to_utf8(
    pCOS *p,
    const char *utf16string,
    int len,
    int *size);

/* Deprecated, use pCOS_convert_to_unicode().
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_utf16_to_utf32(
    pCOS *p,
    const char *utf16string,
    int len,
    const char *ordering,
    int *size);

/* Deprecated, use pCOS_convert_to_unicode().
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_utf32_to_utf8(
    pCOS *p,
    const char *utf32string,
    int len,
    int *size);

/* Deprecated, use pCOS_convert_to_unicode().
*/
PDFLIB_API const char * PDFLIB_CALL
pCOS_utf32_to_utf16(
    pCOS*p,
    const char *utf32string,
    int len,
    const char *ordering,
    int *size);

/*
 * ----------------------------------------------------------------------
 * pCOS API structure with function pointers to all API functions
 * ----------------------------------------------------------------------
 */
#include <setjmp.h>

typedef struct
{
    jmp_buf     jbuf;
} pcos_jmpbuf;

struct pCOS_api_s
{
    /* version numbers for checking the DLL against client code */
    size_t sizeof_pCOS_api; /* size of this structure */

    int major; /* pCOS major version number */
    int minor; /* pCOS minor version number */
    int revision; /* pCOS revision number */

    int reserved; /* reserved */

    void (PDFLIB_CALL * pCOS_close_document)(pCOS *p, int doc);
    const char * (PDFLIB_CALL * pCOS_convert_to_unicode)(pCOS *p,
                const char *inputformat, const char *inputstring, int inputlen,
                int *outputlen, const char *optlist);
    void (PDFLIB_CALL * pCOS_create_pvf)(pCOS *p, const char *filename,
            int len, const void *data, size_t size, const char *optlist);
    const char * (PDFLIB_CALL * pCOS_deflate_unicode)(pCOS *p,
            const char *utfstring, int len, int charsize, int *highchar);
    void (PDFLIB_CALL * pCOS_delete)(pCOS *p);
    int (PDFLIB_CALL * pCOS_delete_pvf)(pCOS *p, const char *filename, int len);
    const pCOS_api * (PDFLIB_CALL * pCOS_get_api)(void);
    const char * (PDFLIB_CALL * pCOS_get_apiname)(pCOS *p);
    const char * (PDFLIB_CALL * pCOS_get_errmsg)(pCOS *p);
    int (PDFLIB_CALL * pCOS_get_errnum)(pCOS *p);
    void * (PDFLIB_CALL * pCOS_get_opaque)(pCOS *p);
    pCOS * (PDFLIB_CALL * pCOS_new)(void);
    pCOS * (PDFLIB_CALL * pCOS_new2)(pcos_error_fp errorhandler, void *opaque);
    int (PDFLIB_CALL * pCOS_open_document)(pCOS *p, const char *filename,
            int len, const char *optlist);
    int (PDFLIB_CALL * pCOS_open_document_mem)(pCOS *p, const void *data,
            size_t size, const char *optlist);
    int (PDFLIB_CALL * pCOS_open_document_callback)(pCOS *p, void *opaque,
            size_t filesize, size_t(*readproc)(void *opaque, void *buffer,
                    size_t size), int(*seekproc)(void *opaque, long offset),
            const char *optlist);
    double (PDFLIB_CALL * pCOS_pcos_get_number)(pCOS *p, int doc,
            const char *path, ...);
    double (PDFLIB_CALL * pCOS_get_number)(pCOS *p, int doc, const char *path,
            ...);
    const char * (PDFLIB_CALL * pCOS_pcos_get_string)(pCOS *p, int doc,
            const char *path, ...);
    const char * (PDFLIB_CALL * pCOS_get_string)(pCOS *p, int doc,
            const char *path, ...);
    const unsigned char * (PDFLIB_CALL * pCOS_pcos_get_stream)(pCOS *p, int doc,
            int *length, const char *optlist, const char *path, ...);
    const unsigned char * (PDFLIB_CALL * pCOS_get_stream)(pCOS *p, int doc,
            int *length, const char *optlist, const char *path, ...);
    double (PDFLIB_CALL * pCOS_info_pvf)(pCOS *p, const char *filename, int len,
                const char *keyword);
    void (PDFLIB_CALL * pCOS_set_option)(pCOS *p, const char *optlist);
    const char * (PDFLIB_CALL * pCOS_utf16_to_utf32)(pCOS *p,
            const char *utf16string, int len, const char *ordering, int *size);
    const char * (PDFLIB_CALL * pCOS_utf16_to_utf8)(pCOS *p,
            const char *utf16string, int len, int *size);
    const char * (PDFLIB_CALL * pCOS_utf32_to_utf16)(pCOS*p,
            const char *utf32string, int len, const char *ordering, int *size);
    const char * (PDFLIB_CALL * pCOS_utf32_to_utf8)(pCOS *p,
            const char *utf32string, int len, int *size);
    const char * (PDFLIB_CALL * pCOS_utf8_to_utf16)(pCOS *p,
            const char *utf8string, const char *ordering, int *size);
    const char * (PDFLIB_CALL * pCOS_utf8_to_utf32)(pCOS *p,
            const char *utf8string, const char *ordering, int *size);

    pcos_jmpbuf * (PDFLIB_CALL * pcos_jbuf)(pCOS *p);
    void (PDFLIB_CALL * pcos_exit_try)(pCOS *p);
    int (PDFLIB_CALL * pcos_catch)(pCOS *p);
    void (PDFLIB_CALL * pcos_rethrow)(pCOS *p);
    void (PDFLIB_CALL * pcos_throw)(pCOS *p, const char *parm1,
            const char *parm2, const char *parm3);
};

/*
 * ----------------------------------------------------------------------
 * Exception handling with try/catch implementation
 * ----------------------------------------------------------------------
 */

/* Set up an exception handling frame; must always be paired with pCOS_CATCH().
*/
#define pCOS_TRY(p)		if (setjmp(pcos_jbuf(p)->jbuf) == 0)

/* Inform the exception machinery that a pCOS_TRY() will be left without
   entering the corresponding pCOS_CATCH( ) clause. */

#define pCOS_EXIT_TRY(p)	pcos_exit_try(p)

/* Catch an exception; must always be paired with pCOS_TRY(). */

#define pCOS_CATCH(p)		if (pcos_catch(p))

/* Re-throw an exception to another handler. */

#define pCOS_RETHROW(p)		pcos_rethrow(p)


/*
 * ----------------------------------------------------------------------
 * Private stuff, do not use explicitly but only via the above macros!
 * ----------------------------------------------------------------------
 */

PDFLIB_API pcos_jmpbuf * PDFLIB_CALL
pcos_jbuf(
    pCOS *p);

PDFLIB_API void PDFLIB_CALL
pcos_exit_try(
    pCOS *p);

PDFLIB_API int PDFLIB_CALL
pcos_catch(
    pCOS *p);

PDFLIB_API void PDFLIB_CALL
pcos_rethrow(
    pCOS *p);

PDFLIB_API void PDFLIB_CALL
pcos_throw(
    pCOS *p,
    const char *parm1,
    const char *parm2,
    const char *parm3);

/*
 * ----------------------------------------------------------------------
 * Deprecated functions (should no longer be used)
 * ----------------------------------------------------------------------
 */

#if _MSC_VER >= 1310    /* VS .NET 2003 and later */
#pragma deprecated(pCOS_open_document_mem)
#pragma deprecated(pCOS_get_number)
#pragma deprecated(pCOS_get_string)
#pragma deprecated(pCOS_get_stream)
#pragma deprecated(pCOS_utf16_to_utf8)
#pragma deprecated(pCOS_utf8_to_utf16)
#pragma deprecated(pCOS_utf32_to_utf16)
#pragma deprecated(pCOS_utf16_to_utf32)
#pragma deprecated(pCOS_utf8_to_utf32)
#pragma deprecated(pCOS_utf32_to_utf8)
#endif

#ifdef __cplusplus
}	/* extern "C" */
#endif

#endif /* PCOSLIB_H */
