/*---------------------------------------------------------------------------*
 |          Copyright (c) 1997-2010 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

/* $Id: pcoslibdl.h,v 1.2 2010/09/27 13:03:00 stm Exp $
 *
 * Function prototypes for dynamically loading the pCOS DLL at runtime
 */

#ifdef __cplusplus
typedef struct pCOS_s pCOS_cpp;
#define pCOS pCOS_cpp
#endif

#include "pcoslib.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Notes for using the pCOS DLL loading mechanism:
 *
 * - pCOS_TRY_DL()/pCOS_CATCH_DL() must be used instead of the standard
 *   exception handling macros.
 * - pCOS_new_dl() must be used instead of pCOS_new()/pCOS_new2().
 * - pCOS_delete_dl() must be used instead of pCOS_delete().
 * - pCOS_get_opaque() must not be used.
 */

/* Load the pCOS DLL, and fetch pointers to all exported functions. */
PDFLIB_API const pCOS_api * PDFLIB_CALL
pCOS_new_dl(pCOS **p_pcos);

/* Delete the pCOS object and unload the previously loaded pCOS DLL */
PDFLIB_API void PDFLIB_CALL
pCOS_delete_dl(const pCOS_api *PDFlib, pCOS *pcos);

#define pCOS_TRY_DL(PCOSAPI, PCOSOBJ)	\
    if (PCOSOBJ) { if (setjmp(PCOSAPI->pcos_jbuf(PCOSOBJ)->jbuf) == 0)

/* Inform the exception machinery that a pCOS_TRY_DL() will be left without
   entering the corresponding pCOS_CATCH_DL() clause. */
#define pCOS_EXIT_TRY_DL(PCOSAPI, PCOSOBJ) PCOSAPI->pcos_exit_try(PCOSOBJ)

/* Catch an exception; must always be paired with pCOS_TRY_DL(). */
#define pCOS_CATCH_DL(PCOSAPI, PCOSOBJ) } if (PCOSAPI->pcos_catch(PCOSOBJ))

/* Re-throw an exception to another handler. */
#define pCOS_RETHROW_DL(PCOSAPI, PCOSOBJ) PCOSAPI->pcos_rethrow(PCOSOBJ)

#ifdef __cplusplus
#undef pCOS
}	/* extern "C" */
#endif
