import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.pdflib.pCOS;
import com.pdflib.pCOSException;

/**
 * pCOS sample servlet for dumping PDF information
 * 
 * @version $Id: DumperServlet.java,v 1.3 2010/10/12 13:39:05 rjs Exp $
 */
public class DumperServlet extends HttpServlet {
    private static final long serialVersionUID = 1L;

    /**
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
     *      response)
     */
    protected void doGet(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("text/html; charset=UTF-8");
        PrintWriter writer = response.getWriter();

	final String 	filename = "TET-datasheet.pdf";
	final String	docoptlist = "requiredmode=minimum";
	final String	globaloptlist = "logging {filename=/tmp/pcos-trace.txt classes={filesearch=3} remove}";

	pCOS p = null;

	int	count, pcosmode;
	int	i, doc;
	boolean plainmetadata;
	String	objtype;

        writer.println("<html>");
        writer.println("<head>");
        writer.println("<title>pCOS J2EE Dumper Servlet Example<title>");
        writer.println("</head>");
        writer.println("<body>");
        writer.println("<pre>");

	try
	{
	    p = new pCOS();

	    p.set_option(globaloptlist);

	    /* This is where input files live. Adjust as necessary. */
            ServletContext context = getServletContext();
            String searchpath = context.getRealPath("/WEB-INF/data");
            String optlist;
            optlist = "searchpath={{" + searchpath + "}}";
            p.set_option(optlist);

	    if ((doc = p.open_document(filename, docoptlist)) == -1)
	    {
		throw new Exception("ERROR " + p.get_errmsg());
	    }

	    /* --------- general information (always available) */

	    pcosmode = (int) p.pcos_get_number(doc, "pcosmode");

	    writer.println("   File name: " + p.pcos_get_string(doc,"filename"));

	    writer.println(" PDF version: " +
		p.pcos_get_string(doc, "pdfversionstring"));

	    writer.println("  Encryption: " +
		p.pcos_get_string(doc, "encrypt/description"));

	    writer.println("   Master pw: " +
		((p.pcos_get_number(doc, "encrypt/master") != 0) ? "yes" : "no"));

	    writer.println("     User pw: " +
		((p.pcos_get_number(doc, "encrypt/user") != 0) ? "yes" : "no"));

	    writer.println("Text copying: " +
		((p.pcos_get_number(doc, "encrypt/nocopy") != 0) ? "no" : "yes"));

	    writer.println("  Linearized: " +
		((p.pcos_get_number(doc, "linearized") != 0) ? "yes" : "no"));

	    if (pcosmode == 0)
	    {
		writer.println(
		    "Minimum mode: no more information available\n");
	    }
	    else {

		/* --------- more details (requires at least user password) */

		writer.println("PDF/X status: " + p.pcos_get_string(doc, "pdfx"));

		writer.println("PDF/A status: " + p.pcos_get_string(doc, "pdfa"));

		boolean xfa_present =
		    !p.pcos_get_string(doc, "type:/Root/AcroForm/XFA").equals("null");
		writer.println("    XFA data: " + (xfa_present ? "yes" : "no"));
		    
		writer.println("  Tagged PDF: " +
		    ((p.pcos_get_number(doc, "tagged") != 0) ? "yes" : "no") + "\n");

		writer.println("No. of pages: " +
		    (int) p.pcos_get_number(doc, "length:pages"));

		writer.println(" Page 1 size: width=" +
		     (int) p.pcos_get_number(doc, "pages[0]/width") + ", height=" +
		     (int) p.pcos_get_number(doc, "pages[0]/height"));

		count = (int) p.pcos_get_number(doc, "length:fonts");
		writer.println("No. of fonts: " +  count);

		for (i = 0; i < count; i++)
		{
		    String fonts;

		    fonts = "fonts[" + i + "]/embedded";
		    if (p.pcos_get_number(doc, fonts) != 0)
			writer.print("embedded ");
		    else
			writer.print("unembedded ");

		    fonts = "fonts[" + i + "]/type";
		    writer.print(p.pcos_get_string(doc, fonts) + " font ");
		    fonts = "fonts[" + i + "]/name";
		    writer.println(p.pcos_get_string(doc, fonts));
		}

		writer.println();

		plainmetadata = p.pcos_get_number(doc, "encrypt/plainmetadata") != 0;

		if (pcosmode == 1 && !plainmetadata &&
		       p.pcos_get_number(doc, "encrypt/nocopy") != 0)
		{
		    writer.print(
			"Restricted mode: no more information available");
		}
		else {

		    /* ----- document info keys and XMP metadata (requires master pw) */

		    count = (int) p.pcos_get_number(doc, "length:/Info");

		    for (i = 0; i < count; i++)
		    {
			String info;
			String key;
			int len;

			info = "type:/Info[" + i + "]";
			objtype = p.pcos_get_string(doc, info);

			info = "/Info[" + i + "].key";
			key = p.pcos_get_string(doc, info);
			len = 12 - key.length();
			while (len-- > 0) writer.print(" ");

			writer.print(key + ": ");

			/* Info entries can be stored as string or name objects */
			if (objtype.equals("name") || objtype.equals("string"))
			{
			    info = "/Info[" + i + "]";
			    writer.println("'" + p.pcos_get_string(doc, info) + "'");
			}
			else
			{
			    info = "type:/Info[" + i + "]";
			    writer.println("(" + p.pcos_get_string(doc, info) +
				 " object)");
			}
		    }

		    writer.print("\n" + "XMP meta data: ");

		    objtype = p.pcos_get_string(doc, "type:/Root/Metadata");
		    if (objtype.equals("stream"))
		    {
			byte[] contents;
			int len;

			contents = p.pcos_get_stream(doc, "", "/Root/Metadata");
			writer.print(contents.length + " bytes ");
			writer.println("");
		    }
		    else
		    {
			writer.println("not present");
		    }
		}
	    }

	    p.close_document(doc);
	}
	catch (pCOSException e)
        {
            System.err.print("pCOS exception occurred in dumper:\n");
            System.err.print("[" + e.get_errnum() + "] " + e.get_apiname() +
                            ": " + e.get_errmsg() + "\n");
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
        }
        finally
        {
            if (p != null)
            {
                p.delete();
            }
        }
    }
}
