# pCOS sample application for dumping PDF information
#
# $Id: dumper.pl,v 1.13 2013/08/02 12:24:34 rjs Exp $

use PDFlib::pCOS 4.0;
use strict;


if (scalar @ARGV != 1)
{   
    die "usage: dumper <filename>";
}


eval {
    my $p = new PDFlib::pCOS;

    my ($count, $pcosmode, $plainmetadata);
    my ($objtype, $i, $doc);
    my $docoptlist = "requiredmode=minimum";
    my $globaloptlist = "";

    $p->set_option($globaloptlist);

    if (($doc = $p->open_document($ARGV[0], $docoptlist)) == -1)
    {
	die("ERROR: %s\n", $p->get_errmsg());
    }

    # --------- general information (always available)

    $pcosmode = $p->pcos_get_number($doc, "pcosmode");

    printf "    File name: %s\n", $p->pcos_get_string($doc, "filename");

    printf "  PDF version: %s\n",
	$p->pcos_get_string($doc, "pdfversionstring");

    printf "   Encryption: %s\n",
	$p->pcos_get_string($doc, "encrypt/description");

    printf "    Master pw: %s\n",
	$p->pcos_get_number($doc, "encrypt/master") ? "yes" : "no";

    printf "      User pw: %s\n",
	$p->pcos_get_number($doc, "encrypt/user") ? "yes" : "no";

    printf "Attachment pw: %s\n",
	$p->pcos_get_number($doc, "encrypt/attachment") ? "yes" : "no";

    printf " Text copying: %s\n",
	$p->pcos_get_number($doc, "encrypt/nocopy") ? "no" : "yes";

    printf "   Linearized: %s\n",
	$p->pcos_get_number($doc, "linearized") ? "yes" : "no";

    if ($pcosmode == 0)
    {
	printf "Minimum mode: no more information available\n\n";
	exit(0);
    }

    # --------- more details (requires at least user password)

    printf " PDF/X status: %s\n", $p->pcos_get_string($doc, "pdfx");

    printf " PDF/A status: %s\n", $p->pcos_get_string($doc, "pdfa");

    printf("     XFA data: %s\n",
        $p->pcos_get_string($doc, "type:/Root/AcroForm/XFA") ne "null"
                            ? "yes" : "no");
                            
    printf "   Tagged PDF: %s\n\n",
	$p->pcos_get_number($doc, "tagged") ? "yes" : "no";

    printf " No. of pages: %d\n",
	$p->pcos_get_number($doc, "length:pages");

    printf "  Page 1 size: width=%g, height=%g\n",
	$p->pcos_get_number($doc, "pages[0]/width"),
	$p->pcos_get_number($doc, "pages[0]/height");

    $count = $p->pcos_get_number($doc, "length:fonts");
    printf " No. of fonts: %d\n", $count;

    for ($i = 0; $i < $count; $i += 1)
    {
	if ($p->pcos_get_number($doc, "fonts[$i]/embedded"))
	{
	    printf "embedded ";
	}
	else
	{
	    printf "unembedded ";
	}

	printf "%s font ", $p->pcos_get_string($doc, "fonts[$i]/type");
	printf "%s\n", $p->pcos_get_string($doc, "fonts[$i]/name");
    }

    printf "\n";

    $plainmetadata = $p->pcos_get_number($doc, "encrypt/plainmetadata");

    if ($pcosmode == 1 && !$plainmetadata &&
	($p->pcos_get_number($doc, "encrypt/nocopy")))
    {
	printf "Restricted mode: no more information available\n\n";
	exit 0;
    }

    # --------- document info keys and XMP metadata (requires master pw
    # or plaintext metadata)
    #

    $count = $p->pcos_get_number($doc, "length:/Info");

    for ($i = 0; $i < $count; $i++)
    {
	$objtype = $p->pcos_get_string($doc, "type:/Info[$i]");
	printf "%13s: ", $p->pcos_get_string($doc, "/Info[$i].key");

	# Info entries can be stored as string or name objects
	if ($objtype eq "string" || $objtype eq "name")
	{
	    printf "'%s'\n", $p->pcos_get_string($doc, "/Info[$i]");
	}
	else
	{
	    printf "(%s object)\n", $p->pcos_get_string($doc, "type:/Info[$i]");
	}
    }

    printf "\nXMP meta data: ";

    $objtype = $p->pcos_get_string($doc, "type:/Root/Metadata");
    if ($objtype eq "stream")
    {
	my $contents;

	$contents = $p->pcos_get_stream($doc, "", "/Root/Metadata");
	printf "%d bytes ", length($contents);

	# This demonstrates Unicode conversion
	my $unicode_chars = 
	    $p->convert_to_unicode("utf8", $contents, "outputformat=utf16");
	printf "(%d Unicode characters)\n\n", length($unicode_chars) / 2;
    }
    else
    {
	printf "not present\n\n";
    }

    $p->close_document($doc);
};

if ($@)
{
    die("$0: pCOS Exception occurred:\n$@");
}
