# pCOS sample application for dumping PDF information with pCOS
#
# $Id: dumper.py,v 1.10 2013/08/02 15:03:59 rjs Exp $
#

from sys import *
from traceback import print_tb, print_exc
from PDFlib.pCOS import *

def yesno(arg):
    if (arg != 0):
        return "yes"
    return "no"


if len(argv) != 2:
    raise Exception("usage: dumper <filename>")


try:
    try:
        pcos = pCOS()

        docoptlist = "requiredmode=minimum"
        globaloptlist = ""

        pcos.set_option(globaloptlist)

        doc = pcos.open_document(argv[1], docoptlist)
        if doc  == -1:
            raise Exception("ERROR: %s\n" % pcos.get_errmsg())

        # --------- general information (always available) */
        pcosmode = int(pcos.pcos_get_number(doc, "pcosmode"))

        print(("    File name: %s" % pcos.pcos_get_string(doc, "filename")))

        print(("  PDF version: %s" % \
            pcos.pcos_get_string(doc, "pdfversionstring")))

        print(("   Encryption: %s" % \
            pcos.pcos_get_string(doc, "encrypt/description")))

        print(("    Master pw: %s" % \
            yesno(pcos.pcos_get_number(doc, "encrypt/master"))))

        print(("      User pw: %s" % \
            yesno(pcos.pcos_get_number(doc, "encrypt/user"))))

        print(("Attachment pw: %s" % \
            yesno(pcos.pcos_get_number(doc, "encrypt/attachment"))))

        print((" Text copying: %s" % \
            yesno(not pcos.pcos_get_number(doc, "encrypt/nocopy"))))

        print(("   Linearized: %s" % \
            yesno(pcos.pcos_get_number(doc, "linearized"))))

        if (pcosmode == 0):
            print("Minimum mode: no more information available\n")
            pcos.close_document(doc)
            exit(0)

        # --------- more details (requires at least user password) */

        print((" PDF/X status: %s" % pcos.pcos_get_string(doc, "pdfx")))

        print((" PDF/A status: %s" % pcos.pcos_get_string(doc, "pdfa")))

        print("     XFA data: %s" % \
            yesno(pcos.pcos_get_number(doc, "type:/Root/AcroForm/XFA") != 0))

        print(("   Tagged PDF: %s\n" % \
            yesno(pcos.pcos_get_number(doc, "tagged"))))

        print((" No. of pages: %d" % \
            int(pcos.pcos_get_number(doc, "length:pages"))))

        print(("  Page 1 size: width=%g, height=%g" % \
            (pcos.pcos_get_number(doc, "pages[%d]/width" % 0),
             pcos.pcos_get_number(doc, "pages[%d]/height" % 0))))

        count = int(pcos.pcos_get_number(doc, "length:fonts"))
        print((" No. of fonts: %d" % count))

        for i in range(count):
            type = pcos.pcos_get_string(doc, "fonts[%d]/type" % i)
            name = pcos.pcos_get_string(doc, "fonts[%d]/name" % i)

            if (pcos.pcos_get_number(doc, "fonts[%d]/embedded" % i)):
                print("embedded %s font %s" % (type, name))
            else:
                print("unembedded %s font %s" % (type, name))

        print

        plainmetadata = \
                int(pcos.pcos_get_number(doc, "encrypt/plainmetadata"))

        if (pcosmode == 1 and not plainmetadata and \
                int(pcos.pcos_get_number(doc, "encrypt/nocopy"))):
            print("Restricted mode: no more information available\n")
            pcos.close_document(doc)
            exit(0)

        # --------- document info keys and XMP metadata (requires master pw
        # or plaintext metadata)
        #

        count = int(pcos.pcos_get_number(doc, "length:/Info"))

        for i in range(0, count):
            objtype = int(pcos.pcos_get_number(doc, "type:/Info[%d]" % i))
            key = pcos.pcos_get_string(doc, "/Info[%d].key" % i)

            # Info entries can be stored as string or name objects */
            # pcos_to_sting == 4; pcos_ot_name == 3
            if (objtype == 4 or objtype == 3):
                print("%13s: '%s'" % \
                    (key, pcos.pcos_get_string(doc, "/Info[%d]" % i)))
            else:
                print("%13s: (%s object)" % \
                   (key, pcos.pcos_get_string(doc, "type:/Info[%d]" % i)))

        print

        objtype = int(pcos.pcos_get_number(doc, "type:/Root/Metadata"))
        # pcos_ot_stream == 7
        if (objtype == 7):
            contents = pcos.pcos_get_stream(doc, "", "/Root/Metadata")
            ustring = contents.decode('utf_8')
            print("XMP meta data: %d bytes (%d Unicode characters)\n" % 
                (len(contents), len(ustring)))
        else:
            print("XMP meta data: not present\n")

        pcos.close_document(doc)

    except pCOSlibException:
        print("pCOS exception occurred:\n[%d] %s: %s" %
            ((pcos.get_errnum(pcos)), pCOS_get_apiname(pcos),  pCOS_get_errmsg()))
        print_tb(exc_info()[2])

    except Exception:
        print("Exception occurred: %s" % (exc_info()[0]))
        print_exc()

finally:
    pcos.delete()
